/*
 *  Copyright WSO2 Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.wso2.carbon.appmgt.hostobjects.internal;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osgi.service.component.ComponentContext;
import org.wso2.carbon.appmgt.hostobjects.HostObjectUtils;
import org.wso2.carbon.appmgt.impl.AppMConstants;
import org.wso2.carbon.appmgt.impl.AppManagerConfiguration;
import org.wso2.carbon.appmgt.impl.AppManagerConfigurationService;
import org.wso2.carbon.appmgt.impl.service.TenantConfigurationService;
import org.wso2.carbon.registry.core.service.RegistryService;
import org.wso2.carbon.user.core.service.RealmService;
import org.wso2.carbon.utils.ConfigurationContextService;

import java.util.List;

/**
 * @scr.component name="org.wso2.appmgt.hostobjects" immediate="true"
 *
 * @scr.reference name="api.manager.config.service"
 * interface="org.wso2.carbon.appmgt.impl.AppManagerConfigurationService" cardinality="1..1"
 * policy="dynamic" bind="setAPIManagerConfigurationService" unbind="unsetAPIManagerConfigurationService"
 *
 * @scr.reference name="config.context.service"
 * interface="org.wso2.carbon.utils.ConfigurationContextService"
 * cardinality="1..1"
 * policy="dynamic"
 * bind="setConfigurationContextService"
 * unbind="unsetConfigurationContextService"
 *
 * @scr.reference name="user.realm.service"
 * interface="org.wso2.carbon.user.core.service.RealmService"
 * cardinality="1..1" policy="dynamic" bind="setRealmService" unbind="unsetRealmService"
 *
 * @scr.reference name="registry.service"
 * interface="org.wso2.carbon.registry.core.service.RegistryService"
 * cardinality="1..1" policy="dynamic" bind="setRegistryService" unbind="unsetRegistryService"
 *
 * @scr.reference name="org.wso2.carbon.appmgt.impl.service.TenantConfigurationService"
 * interface="org.wso2.carbon.appmgt.impl.service.TenantConfigurationService" cardinality="1..1"
 * policy="dynamic" bind="setTenantConfigurationService" unbind="unsetTenantConfigurationService
 *
 */
public class HostObjectComponent {

    private static final Log log = LogFactory.getLog(HostObjectComponent.class);

    private static AppManagerConfiguration configuration = null;

    protected void activate(ComponentContext componentContext) {
       if (log.isDebugEnabled()){
           log.debug("HostObjectComponent activated");
       }
    }

    protected void deactivate(ComponentContext componentContext) {
       if (log.isDebugEnabled()){
           log.debug("HostObjectComponent deactivated");
       }
    }

    protected void setAPIManagerConfigurationService(AppManagerConfigurationService amcService) {
        if (log.isDebugEnabled()) {
            log.debug("WebApp manager configuration service bound to the WebApp host objects");
        }
        configuration = amcService.getAPIManagerConfiguration();
        ServiceReferenceHolder.getInstance().setAPIManagerConfigurationService(amcService);
    }

    protected void unsetAPIManagerConfigurationService(AppManagerConfigurationService amcService) {
        if (log.isDebugEnabled()) {
            log.debug("WebApp manager configuration service unbound from the WebApp host objects");
        }
        configuration = null;
        ServiceReferenceHolder.getInstance().setAPIManagerConfigurationService(null);
    }

    public static AppManagerConfiguration getAPIManagerConfiguration() {
        return configuration;
    }

     protected void setConfigurationContextService(ConfigurationContextService configCtx) {
        HostObjectUtils.setConfigContextService(configCtx);
    }

    protected void unsetConfigurationContextService(ConfigurationContextService configCtx) {
        HostObjectUtils.setConfigContextService(null);
    }

    protected void setTenantConfigurationService(TenantConfigurationService tenantConfigurationService) {
        if (log.isDebugEnabled()) {
            log.debug("Setting tenant configuration service implementation - " + tenantConfigurationService.getClass().getName());
        }
        ServiceReferenceHolder.getInstance().setTenantConfigurationService(tenantConfigurationService);
    }

    protected void unsetTenantConfigurationService(TenantConfigurationService tenantConfigurationService) {
        if (log.isDebugEnabled()) {
            log.debug("Un-setting tenant configuration service implementation - " + tenantConfigurationService.getClass().getName());
        }
        ServiceReferenceHolder.getInstance().setTenantConfigurationService(null);
    }

    protected void setRealmService(RealmService realmService) {
        if (realmService != null && log.isDebugEnabled()) {
            log.debug("Realm service initialized");
        }
        ServiceReferenceHolder.getInstance().setRealmService(realmService);
    }

    protected void unsetRealmService(RealmService realmService) {
        ServiceReferenceHolder.getInstance().setRealmService(null);
    }

    protected void setRegistryService(RegistryService registryService) {
        if (registryService != null && log.isDebugEnabled()) {
            log.debug("Registry service initialized");
        }
        ServiceReferenceHolder.getInstance().setRegistryService(registryService);
    }

    protected void unsetRegistryService(RegistryService registryService) {
        ServiceReferenceHolder.getInstance().setRegistryService(null);
    }

    public static boolean isDisplayMultipleVersionsEnabled() {
        return Boolean.parseBoolean(getAPIManagerConfiguration().getFirstProperty(
                AppMConstants.STORE_DISPLAY_MULTIPLE_VERSIONS));
    }

    public static List<String> getEnabledAssetTypeList() {
        return getAPIManagerConfiguration().getProperty(AppMConstants.ENABLED_ASSET_TYPE);
    }
}
